@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteft2.c,v 1.6 2003/05/06 07:38:39 nalin Exp $"
 
 #include "../config.h"
 
@@ -56,7 +55,7 @@ static void
 _vte_ft2_create(struct _vte_draw *draw, GtkWidget *widget)
 {
 	struct _vte_ft2_data *data;
-	data = (struct _vte_ft2_data*) g_malloc0(sizeof(struct _vte_ft2_data));
+	data = g_slice_new0(struct _vte_ft2_data);
 	draw->impl_data = data;
 	data->rgb = NULL;
 	memset(&data->color, 0, sizeof(data->color));
@@ -82,7 +81,7 @@ _vte_ft2_destroy(struct _vte_draw *draw)
 		data->pixbuf = NULL;
 	}
 	data->scrollx = data->scrolly = 0;
-	g_free(data);
+	g_slice_free(struct _vte_ft2_data, data);
 }
 
 static GdkVisual *
@@ -133,7 +132,7 @@ _vte_ft2_end(struct _vte_draw *draw)
 	    (data->bottom == -1)) {
 		_vte_rgb_draw_on_drawable(widget->window,
 					  widget->style->fg_gc[state],
-					  0, 0, 
+					  0, 0,
 					  width, height,
 					  data->rgb,
 					  0, 0);
@@ -166,10 +165,14 @@ _vte_ft2_set_background_image(struct _vte_draw *draw,
 {
 	struct _vte_ft2_data *data;
 	GdkPixbuf *bgpixbuf;
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen(draw->widget);
 
 	data = (struct _vte_ft2_data*) draw->impl_data;
 
-	bgpixbuf = vte_bg_get_pixbuf(vte_bg_get(), type, pixbuf, file,
+	bgpixbuf = vte_bg_get_pixbuf(vte_bg_get_for_screen(screen),
+				     type, pixbuf, file,
 				     color, saturation);
 	if (GDK_IS_PIXBUF(data->pixbuf)) {
 		g_object_unref(G_OBJECT(data->pixbuf));
@@ -212,7 +215,8 @@ _vte_ft2_clear(struct _vte_draw *draw,
 
 static void
 _vte_ft2_set_text_font(struct _vte_draw *draw,
-		       const PangoFontDescription *fontdesc)
+		       const PangoFontDescription *fontdesc,
+		       VteTerminalAntiAlias anti_alias)
 {
 	struct _vte_ft2_data *data;
 
@@ -223,7 +227,8 @@ _vte_ft2_set_text_font(struct _vte_draw *draw,
 		data->cache = NULL;
 	}
 	data->cache = _vte_glyph_cache_new();
-	_vte_glyph_cache_set_font_description(NULL, data->cache, fontdesc,
+	_vte_glyph_cache_set_font_description(draw->widget, NULL,
+					      data->cache, fontdesc, anti_alias,
 					      NULL, NULL);
 }
 
@@ -251,6 +256,22 @@ _vte_ft2_get_text_ascent(struct _vte_draw *draw)
 	return data->cache->ascent;
 }
 
+static int
+_vte_ft2_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	struct _vte_ft2_data *data;
+	const struct _vte_glyph *glyph;
+
+	data = (struct _vte_ft2_data*) draw->impl_data;
+
+	glyph = _vte_glyph_get(data->cache, c);
+	if (glyph != NULL) {
+		return glyph->width;
+	}
+
+	return _vte_ft2_get_text_width(draw) * columns;
+}
+
 static gboolean
 _vte_ft2_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -279,6 +300,24 @@ _vte_ft2_draw_text(struct _vte_draw *draw,
 	}
 }
 
+static gboolean
+_vte_ft2_draw_char(struct _vte_draw *draw,
+		   struct _vte_draw_text_request *request,
+		   GdkColor *color, guchar alpha)
+{
+	struct _vte_ft2_data *data;
+
+	data = (struct _vte_ft2_data*) draw->impl_data;
+
+	if (data->cache != NULL) {
+		if (_vte_glyph_get(data->cache, request->c) != NULL) {
+			_vte_ft2_draw_text(draw, request, 1, color, alpha);
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static void
 _vte_ft2_draw_rectangle(struct _vte_draw *draw,
 			gint x, gint y, gint width, gint height,
@@ -340,13 +379,16 @@ struct _vte_draw_impl _vte_draw_ft2 = {
 	_vte_ft2_end,
 	_vte_ft2_set_background_color,
 	_vte_ft2_set_background_image,
+	FALSE,
 	_vte_ft2_clear,
 	_vte_ft2_set_text_font,
 	_vte_ft2_get_text_width,
 	_vte_ft2_get_text_height,
 	_vte_ft2_get_text_ascent,
+	_vte_ft2_get_char_width,
 	_vte_ft2_get_using_fontconfig,
 	_vte_ft2_draw_text,
+	_vte_ft2_draw_char,
 	_vte_ft2_draw_rectangle,
 	_vte_ft2_fill_rectangle,
 	_vte_ft2_set_scroll,
