@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteskel.c,v 1.10 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -48,7 +47,7 @@ _vte_skel_create(struct _vte_draw *draw, GtkWidget *widget)
 {
 	struct _vte_skel_data *data;
 
-	draw->impl_data = g_malloc(sizeof(struct _vte_skel_data));
+	draw->impl_data = g_slice_new(struct _vte_skel_data);
 	data = (struct _vte_skel_data*) draw->impl_data;
 
 	data->color.red = 0;
@@ -75,7 +74,7 @@ _vte_skel_destroy(struct _vte_draw *draw)
 
 	memset(&data->color, 0, sizeof(data->color));
 
-	g_free(draw->impl_data);
+	g_slice_free(struct _vte_skel_data, draw->impl_data);
 }
 
 static GdkVisual *
